/* eslint-disable react/display-name */
import { Accordion } from '@material-ui/core'
import Button from '@material-ui/core/Button'
import AnalysisConsumption from 'components/Analysis/AnalysisConsumption'
import { FluidType } from 'enum/fluid.enum'
import { mount } from 'enzyme'
import { DateTime } from 'luxon'
import { PerformanceIndicator } from 'models'
import React from 'react'
import { act } from 'react-dom/test-utils'
import { Provider } from 'react-redux'
import configureStore from 'redux-mock-store'
import { globalStateData } from '../../../tests/__mocks__/globalStateData.mock'
import { profileData } from '../../../tests/__mocks__/profileData.mock'
import {
  mockMonthlyForecastJanuaryTestProfile1,
  profileTypeData,
} from '../../../tests/__mocks__/profileType.mock'

jest.mock('cozy-ui/transpiled/react/I18n', () => {
  return {
    useI18n: jest.fn(() => {
      return {
        t: (str: string) => str,
      }
    }),
  }
})

const mockedNavigate = jest.fn()
jest.mock('react-router-dom', () => ({
  ...jest.requireActual('react-router-dom'),
  useNavigate: () => mockedNavigate,
}))

const mockgetMonthlyForecast = jest.fn()
jest.mock('services/profileType.service', () => {
  return jest.fn(() => {
    return {
      getMonthlyForecast: mockgetMonthlyForecast,
    }
  })
})

const mockgetProfileTypeData = jest.fn()
jest.mock('services/profileTypeEntity.service', () => {
  return jest.fn(() => {
    return {
      getProfileType: mockgetProfileTypeData,
    }
  })
})

jest.mock('components/Analysis/AnalysisConsumptionRow', () => () => (
  <div id="analysisconsumptionrow" />
))

const mockStore = configureStore([])

const modifiedProfile = { ...profileData, isProfileTypeCompleted: true }

const performanceIndicator = {
  compareValue: 160.42797399999998,
  percentageVariation: 0.026592126632478563,
  value: 156.161853,
}
const performanceIndicators = [
  {
    compareValue: 203.49,
    percentageVariation: 0.12261044768784712,
    value: 178.54,
  },
  {
    compareValue: 7926.82,
    percentageVariation: 0.020542916327102145,
    value: 7763.98,
  },
  {
    compareValue: 1316.46,
    percentageVariation: -0.0009191316105312541,
    value: 1317.67,
  },
]

describe('AnalysisConsumption component', () => {
  it('should be rendered correctly', async () => {
    const store = mockStore({
      ecolyo: {
        profile: modifiedProfile,
        profileType: profileTypeData,
        global: globalStateData,
      },
    })
    const wrapper = mount(
      <Provider store={store}>
        <AnalysisConsumption
          aggregatedPerformanceIndicator={performanceIndicator}
          performanceIndicators={performanceIndicators}
          analysisDate={DateTime.fromISO('2021-07-01T00:00:00.000Z', {
            zone: 'utc',
          })}
        />
      </Provider>
    )
    await act(async () => {
      await new Promise(resolve => setTimeout(resolve))
      wrapper.update()
    })
    expect(wrapper.find(Accordion).exists()).toBeTruthy()
    expect(wrapper.find('#analysisconsumptionrow').length).toBe(4)
  })

  it('should be rendered correctly with no profil set', async () => {
    const store = mockStore({
      ecolyo: {
        profile: profileData,
        profileType: profileTypeData,
        global: globalStateData,
      },
    })
    const mockAggregatedPerformanceIndicator: PerformanceIndicator = {
      compareValue: 160.42797399999998,
      percentageVariation: null,
      value: null,
    }
    const wrapper = mount(
      <Provider store={store}>
        <AnalysisConsumption
          aggregatedPerformanceIndicator={mockAggregatedPerformanceIndicator}
          performanceIndicators={performanceIndicators}
          analysisDate={DateTime.fromISO('2021-07-01T00:00:00.000Z', {
            zone: 'utc',
          })}
        />
      </Provider>
    )
    await act(async () => {
      await new Promise(resolve => setTimeout(resolve))
      wrapper.update()
    })
    expect(wrapper.find(Button).exists()).toBeTruthy()
  })

  it('should be rendered correctly with null aggregated performance indicator', async () => {
    const store = mockStore({
      ecolyo: {
        profile: modifiedProfile,
        profileType: profileTypeData,
        global: globalStateData,
      },
    })
    const mockAggregatedPerformanceIndicator: PerformanceIndicator = {
      compareValue: 160.42797399999998,
      percentageVariation: null,
      value: null,
    }
    const wrapper = mount(
      <Provider store={store}>
        <AnalysisConsumption
          aggregatedPerformanceIndicator={mockAggregatedPerformanceIndicator}
          performanceIndicators={performanceIndicators}
          analysisDate={DateTime.fromISO('2021-07-01T00:00:00.000Z', {
            zone: 'utc',
          })}
        />
      </Provider>
    )
    await act(async () => {
      await new Promise(resolve => setTimeout(resolve))
      wrapper.update()
    })
    expect(wrapper.find(Accordion).exists()).toBeTruthy()
    expect(wrapper.find('#analysisconsumptionrow').length).toBe(4)
  })

  it('should be rendered correctly without fluid', async () => {
    const store = mockStore({
      ecolyo: {
        profile: modifiedProfile,
        profileType: profileTypeData,
        global: globalStateData,
      },
    })
    const wrapper = mount(
      <Provider store={store}>
        <AnalysisConsumption
          aggregatedPerformanceIndicator={performanceIndicator}
          performanceIndicators={performanceIndicators}
          analysisDate={DateTime.fromISO('2021-07-01T00:00:00.000Z', {
            zone: 'utc',
          })}
        />
      </Provider>
    )
    await act(async () => {
      await new Promise(resolve => setTimeout(resolve))
      wrapper.update()
    })
    expect(
      wrapper.find('#analysisconsumptionrow').first().parent().prop('fluid')
    ).toBe(FluidType.MULTIFLUID)
  })

  it('should be rendered correctly with all fluids connected', async () => {
    const updateGlobalState = { ...globalStateData }
    updateGlobalState.fluidTypes = [
      FluidType.ELECTRICITY,
      FluidType.WATER,
      FluidType.GAS,
    ]
    const store = mockStore({
      ecolyo: {
        profile: modifiedProfile,
        profileType: profileTypeData,
        global: updateGlobalState,
      },
    })
    mockgetMonthlyForecast.mockReturnValue(
      mockMonthlyForecastJanuaryTestProfile1
    )
    const wrapper = mount(
      <Provider store={store}>
        <AnalysisConsumption
          aggregatedPerformanceIndicator={performanceIndicator}
          performanceIndicators={performanceIndicators}
          analysisDate={profileData.monthlyAnalysisDate}
        />
      </Provider>
    )
    await act(async () => {
      await new Promise(resolve => setTimeout(resolve))
      wrapper.update()
    })
    expect(mockgetMonthlyForecast).toHaveBeenCalledWith(
      profileData.monthlyAnalysisDate.month - 1
    )
    expect(
      wrapper.find('#analysisconsumptionrow').first().parent().prop('fluid')
    ).toBe(FluidType.MULTIFLUID)
  })

  it('should be rendered correctly with 2 fluids connected', async () => {
    const updateGlobalState = { ...globalStateData }
    updateGlobalState.fluidTypes = [FluidType.ELECTRICITY, FluidType.WATER]
    const store = mockStore({
      ecolyo: {
        profile: modifiedProfile,
        global: updateGlobalState,
      },
    })
    mockgetMonthlyForecast.mockReturnValue(
      mockMonthlyForecastJanuaryTestProfile1
    )
    const wrapper = mount(
      <Provider store={store}>
        <AnalysisConsumption
          aggregatedPerformanceIndicator={performanceIndicator}
          performanceIndicators={performanceIndicators}
          analysisDate={DateTime.fromISO('2020-11-03T00:00:00.000Z', {
            zone: 'utc',
          })}
        />
      </Provider>
    )
    await act(async () => {
      await new Promise(resolve => setTimeout(resolve))
      wrapper.update()
    })
    expect(mockgetMonthlyForecast).toHaveBeenCalledWith(
      profileData.monthlyAnalysisDate.month - 1
    )
    expect(
      wrapper.find('#analysisconsumptionrow').first().parent().prop('fluid')
    ).toBe(FluidType.MULTIFLUID)
  })

  it('should redirect to profileType form when click on mui button', async () => {
    const store = mockStore({
      ecolyo: {
        profile: profileData,
        profileType: profileTypeData,
        global: globalStateData,
      },
    })
    const wrapper = mount(
      <Provider store={store}>
        <AnalysisConsumption
          aggregatedPerformanceIndicator={performanceIndicator}
          performanceIndicators={performanceIndicators}
          analysisDate={DateTime.fromISO('2021-07-01T00:00:00.000Z', {
            zone: 'utc',
          })}
        />
      </Provider>
    )
    await act(async () => {
      await new Promise(resolve => setTimeout(resolve))
      wrapper.update()
    })
    wrapper.find(Button).first().simulate('click')
    expect(mockedNavigate).toBeCalledWith('/profileType')
  })
})
