import { challengeReducer } from './challenge.reducer'
import {
  SET_USER_CHALLENGE_LIST,
  UPDATE_USER_CHALLENGE_LIST,
  UNLOCK_NEXT_USER_CHALLENGE,
  SET_CHALLENGE_CONSUMPTION,
} from './challenge.actions'
import { Dataload, ChallengeState, UserChallenge } from 'models'
import { UserChallengeState } from 'enum/userChallenge.enum'
import {
  userChallengeData,
  userChallengeDefault,
} from '../../../tests/__mocks__/userChallengeData.mock'
import { DateTime } from 'luxon'
import { mockInitialChallengeState } from '../../../tests/__mocks__/store'

describe('challenge reducer', () => {
  it('should return the initial state', () => {
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    const result = challengeReducer(undefined as any, { type: 'default' })
    expect(result).toEqual(mockInitialChallengeState)
  })

  it('should handle SET_USER_CHALLENGE_LIST with payload', () => {
    const result = challengeReducer(mockInitialChallengeState, {
      type: SET_USER_CHALLENGE_LIST,
      payload: userChallengeData,
    })
    const expectedResult: ChallengeState = {
      userChallengeList: userChallengeData,
      currentChallenge: userChallengeData[2],
      currentDataload: [],
    }
    expect(result).toEqual(expectedResult)
  })

  it('should handle SET_USER_CHALLENGE_LIST without payload', () => {
    const result = challengeReducer(mockInitialChallengeState, {
      type: SET_USER_CHALLENGE_LIST,
    })
    expect(result).toEqual(mockInitialChallengeState)
  })

  it('should handle UPDATE_USER_CHALLENGE_LIST with payload', () => {
    const updatedMockInitialChallengeState = {
      ...mockInitialChallengeState,
      userChallengeList: userChallengeDefault,
    }
    const updatedUserChallenge: UserChallenge = {
      ...userChallengeDefault[0],
      state: UserChallengeState.ONGOING,
    }
    const result = challengeReducer(updatedMockInitialChallengeState, {
      type: UPDATE_USER_CHALLENGE_LIST,
      payload: updatedUserChallenge,
    })
    const expectedResult: ChallengeState = {
      userChallengeList: [
        updatedUserChallenge,
        ...userChallengeDefault.slice(1),
      ],
      currentChallenge: updatedUserChallenge,
      currentDataload: [],
    }
    expect(result).toEqual(expectedResult)
  })

  it('should handle UPDATE_USER_CHALLENGE_LIST without payload', () => {
    const result = challengeReducer(mockInitialChallengeState, {
      type: UPDATE_USER_CHALLENGE_LIST,
    })
    expect(result).toEqual(mockInitialChallengeState)
  })

  it('should handle UNLOCK_NEXT_USER_CHALLENGE with payload', () => {
    const updatedMockInitialChallengeState = {
      ...mockInitialChallengeState,
      userChallengeList: userChallengeDefault,
    }
    const updatedUserChallenge: UserChallenge = {
      ...userChallengeDefault[0],
      state: UserChallengeState.DONE,
    }
    const unlockedUserChallenge: UserChallenge = {
      ...userChallengeDefault[1],
      state: UserChallengeState.UNLOCKED,
    }
    const result = challengeReducer(updatedMockInitialChallengeState, {
      type: UNLOCK_NEXT_USER_CHALLENGE,
      payload: updatedUserChallenge,
    })
    const expectedResult: ChallengeState = {
      userChallengeList: [
        updatedUserChallenge,
        unlockedUserChallenge,
        ...userChallengeDefault.slice(2, 6),
      ],
      currentChallenge: null,
      currentDataload: [],
    }
    expect(result).toEqual(expectedResult)
  })

  it('should handle UNLOCK_NEXT_USER_CHALLENGE without payload', () => {
    const result = challengeReducer(mockInitialChallengeState, {
      type: UNLOCK_NEXT_USER_CHALLENGE,
    })
    expect(result).toEqual(mockInitialChallengeState)
  })

  it('should handle SET_CHALLENGE_CONSUMPTION with payload', () => {
    const updatedMockInitialChallengeState = {
      ...mockInitialChallengeState,
      userChallengeList: userChallengeDefault,
    }
    const dataload: Dataload[] = [
      {
        date: DateTime.fromISO('2020-10-01T00:00:00.000Z', {
          zone: 'utc',
        }),
        value: 20,
        valueDetail: [5.0, 10.0, 5.0],
      },
    ]
    const updatedUserChallenge: UserChallenge = {
      ...userChallengeDefault[0],
      duel: {
        ...userChallengeDefault[0].duel,
        userConsumption: 20,
      },
    }
    const expectedResult: ChallengeState = {
      userChallengeList: [
        updatedUserChallenge,
        ...userChallengeDefault.slice(1, 6),
      ],
      currentChallenge: null,
      currentDataload: dataload,
    }
    const result = challengeReducer(updatedMockInitialChallengeState, {
      type: SET_CHALLENGE_CONSUMPTION,
      payload: {
        userChallenge: updatedUserChallenge,
        currentDataload: dataload,
      },
    })
    expect(result).toEqual(expectedResult)
  })

  it('should handle SET_CHALLENGE_CONSUMPTION without payload', () => {
    const result = challengeReducer(mockInitialChallengeState, {
      type: SET_CHALLENGE_CONSUMPTION,
    })
    expect(result).toEqual(mockInitialChallengeState)
  })
})
